@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2017-2019 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2017 Richard Hughes <richard@hughsie.com>
  *
  * SPDX-License-Identifier: LGPL-2.1+
  */
@@ -23,16 +23,7 @@ G_DECLARE_DERIVABLE_TYPE (FuUdevDevice, fu_udev_device, FU, UDEV_DEVICE, FuDevic
 struct _FuUdevDeviceClass
 {
 	FuDeviceClass	parent_class;
-	gboolean	 (*probe)			(FuUdevDevice	*device,
-							 GError		**error);
-	gboolean	 (*open)			(FuUdevDevice	*device,
-							 GError		**error);
-	gboolean	 (*close)			(FuUdevDevice	*device,
-							 GError		**error);
-	void		 (*to_string)			(FuUdevDevice	*self,
-							 guint		 indent,
-							 GString	*str);
-	gpointer	__reserved[28];
+	gpointer	__reserved[31];
 };
 
 /**
@@ -42,6 +33,7 @@ struct _FuUdevDeviceClass
  * @FU_UDEV_DEVICE_FLAG_OPEN_WRITE:		Open the device write-only
  * @FU_UDEV_DEVICE_FLAG_VENDOR_FROM_PARENT:	Get the vendor ID fallback from the parent
  * @FU_UDEV_DEVICE_FLAG_USE_CONFIG:		Read and write from the device config
+ * @FU_UDEV_DEVICE_FLAG_OPEN_NONBLOCK:		Open nonblocking, e.g. O_NONBLOCK
  *
  * Flags used when opening the device using fu_device_open().
  **/
@@ -58,9 +50,12 @@ typedef enum {
 
 FuUdevDevice	*fu_udev_device_new			(GUdevDevice	*udev_device);
 GUdevDevice	*fu_udev_device_get_dev			(FuUdevDevice	*self);
+void		 fu_udev_device_set_dev			(FuUdevDevice	*self,
+							 GUdevDevice	*udev_device);
 const gchar	*fu_udev_device_get_device_file		(FuUdevDevice	*self);
 const gchar	*fu_udev_device_get_sysfs_path		(FuUdevDevice	*self);
 const gchar	*fu_udev_device_get_subsystem		(FuUdevDevice	*self);
+const gchar	*fu_udev_device_get_driver		(FuUdevDevice	*self);
 guint32		 fu_udev_device_get_vendor		(FuUdevDevice	*self);
 guint32		 fu_udev_device_get_model		(FuUdevDevice	*self);
 guint32		 fu_udev_device_get_subsystem_vendor	(FuUdevDevice	*self);
@@ -71,7 +66,12 @@ guint		 fu_udev_device_get_slot_depth		(FuUdevDevice	*self,
 							 const gchar	*subsystem);
 gboolean	 fu_udev_device_set_physical_id		(FuUdevDevice	*self,
 							 const gchar	*subsystems,
-							 GError		**error);
+							 GError		**error)
+							 G_GNUC_WARN_UNUSED_RESULT;
+gboolean	 fu_udev_device_set_logical_id		(FuUdevDevice	*self,
+							 const gchar	*subsystem,
+							 GError		**error)
+							 G_GNUC_WARN_UNUSED_RESULT;
 void		 fu_udev_device_set_readonly		(FuUdevDevice	*self,
 							 gboolean	 readonly)
 G_GNUC_DEPRECATED_FOR(fu_udev_device_set_flags);
@@ -85,25 +85,30 @@ gboolean	 fu_udev_device_ioctl			(FuUdevDevice	*self,
 							 gulong		 request,
 							 guint8		*buf,
 							 gint		*rc,
-							 GError		**error);
+							 GError		**error)
+							 G_GNUC_WARN_UNUSED_RESULT;
 gboolean	 fu_udev_device_pwrite			(FuUdevDevice	*self,
 							 goffset	 port,
 							 guint8		 data,
-							 GError		**error);
+							 GError		**error)
+							 G_GNUC_WARN_UNUSED_RESULT;
 gboolean	 fu_udev_device_pwrite_full		(FuUdevDevice	*self,
 							 goffset	 port,
 							 const guint8	*buf,
 							 gsize		 bufsz,
-							 GError		**error);
+							 GError		**error)
+							 G_GNUC_WARN_UNUSED_RESULT;
 gboolean	 fu_udev_device_pread			(FuUdevDevice	*self,
 							 goffset	 port,
 							 guint8		*data,
-							 GError		**error);
+							 GError		**error)
+							 G_GNUC_WARN_UNUSED_RESULT;
 gboolean	 fu_udev_device_pread_full		(FuUdevDevice	*self,
 							 goffset	 port,
 							 guint8		*buf,
 							 gsize		 bufsz,
-							 GError		**error);
+							 GError		**error)
+							 G_GNUC_WARN_UNUSED_RESULT;
 const gchar	*fu_udev_device_get_sysfs_attr		 (FuUdevDevice	*self,
 							  const gchar	*attr,
 							  GError	**error);
@@ -112,5 +117,10 @@ gchar		*fu_udev_device_get_parent_name		(FuUdevDevice	*self);
 gboolean	 fu_udev_device_write_sysfs		(FuUdevDevice	*self,
 							 const gchar	*attribute,
 							 const gchar	*val,
-							 GError		**error);
+							 GError		**error)
+							 G_GNUC_WARN_UNUSED_RESULT;
 const gchar	*fu_udev_device_get_devtype		(FuUdevDevice	*self);
+GPtrArray 	*fu_udev_device_get_siblings_with_subsystem (FuUdevDevice *self,
+							     const gchar *subsystem);
+GPtrArray 	*fu_udev_device_get_children_with_subsystem (FuUdevDevice *self,
+							     const gchar *subsystem);
